const { resolve ,join } = require('path')
const vueLoader = require('./lib/vueLoader.js')
const babelPreset = require('./lib/babelPreset.js')
const styleLoader = require('./lib/styleLoader.js')
const htmlPlugin = require('html-webpack-plugin-for-multihtml')
const progressBarPlugin = require('progress-bar-webpack-plugin')
const friendlyErrorsWebpackPlugin = require('friendly-errors-webpack-plugin')
const HtmlWebpackInlineSourcePlugin = require('html-webpack-inline-source-plugin');
const AutoDllPlugin = require('autodll-webpack-plugin')
const UglifyJsPlugin = require('uglifyjs-webpack-plugin')
const webpack = require('webpack')
const fse = require('fs-extra')

const srcDir = resolve(process.cwd(),'src')

exports.resolve = (isProduction)=> {
  return {
    modules:[resolve(__dirname,'../../../node_modules'),'node_modules'],
    extensions: ['.js', '.json', '.vue'],
    alias: {
      'vue$': isProduction ? 'vue/dist/vue.min.js' : 'vue/dist/vue.esm.js',
      'commons': resolve(process.cwd(), 'src/commons')
    }
  }
}


exports.getModules = (pageName,isProduction)=>{

  return {
    rules: [
      {
        test: /\.vue$/,
        exclude: /node_modules/,
        use: [{
          loader: 'vue-loader',
          options: vueLoader(isProduction)
        }],
      },
      {
        test: /\.pug$/,
        use: [{
          loader: 'pug-loader',
        }],
      },
      {
        test: /\.html$/,
        use: [{
          loader: 'html-loader',
        }],
      },
      {
        test: /\.yaml$/,
        use: [{
          loader: 'yaml-loader',
        }],
      },
      {
        test: /\.js$/,
        exclude: /node_modules/,
        use: [{
          loader: 'babel-loader',
          options: {
            cacheDirectory:true,
            presets: babelPreset
          }
        }],
      },
      // css-loader 让js识别css文件，style-loader让css嵌入到style标签中
      { test: /\.css$/ , use: styleLoader('css',isProduction)},
      { test: /\.less$/ , use: styleLoader('less',isProduction)},
      {
        test: /\.(png|jpe?g|gif|svg)$/,
        use: [{
          loader: 'url-loader',
          options: {
            limit: 1000, // 1KO
            name: `assets/${pageName}/images/[name].[hash:8].[ext]`
          }
        }]
      },
      {
        test: /\.(woff2?|eot|ttf|otf)(\?.*)?$/,
        use: [{
          loader: 'url-loader',
          options: {
            limit: 1000, // 1 KO
            name: `assets/${pageName}/fonts/[name].[hash:8].[ext]`
          }
        }]
      },
    ]
  }
}

const genHtml = ({ pageName , title } ,entries,isPro)=>{
  const ownTpl = resolve(srcDir,pageName,'index.html')
  const prefix = isPro ? `${pageName}/` : ''
  const res = []
  entries.forEach((name)=>{
    res.push(new htmlPlugin({
      multihtmlCache: true,
      title: title ? title : 'simple',
      filename: name === `index` ? `${prefix}index.html` : `${prefix}${name}/index.html`,
      chunks:[name,'vendor','manifest','common'],
      template: fse.pathExistsSync(ownTpl) ? ownTpl : resolve(__dirname,'../template/index.pug'),
      // inlineSource: /manifest\.(.+)?\.js$/ ,
      inject: true,
      minify:{
        removeComments: true,
        collapseWhitespace: true,
        minifyCSS: true,
        minifyJS: true
      },
    }))
  })
  return res
}

exports.getPlugins = ({pageName,title , env,dll, ...options},entries,isPro)=>{
  let newdll = dll.map((v)=>{
    if(v === 'vue'){
      return isPro ? 'vue/dist/vue.min.js' : 'vue/dist/vue.esm.js'
    }
    return v
  })
  //多页html
  const htmls = genHtml({pageName,title},entries,isPro)
  return htmls.concat([
    //设置资源内链
    // new HtmlWebpackInlineSourcePlugin(),
    new webpack.optimize.CommonsChunkPlugin({
      name: 'common',// 共同模块。
      minChunks (module , count) {
        return count >= 2
      }
    }),
    // new webpack.optimize.CommonsChunkPlugin({
    //   name: 'vendor',// 指定公共 bundle 的名字。
    //   chunks: 'common',//从公共模块中合并第三方库
    //   minChunks (module) {
    //     // any required modules inside node_modules are extracted to vendor
    //     return (
    //       module.resource &&
    //       /\.js$/.test(module.resource) &&
    //       module.resource.indexOf(`node_modules`) > -1
    //     )
    //   }
    // }),
    new webpack.optimize.CommonsChunkPlugin({
      name: 'manifest',// 指定公共 bundle 的名字。
      minChunks: Infinity
    }),
    new webpack.DefinePlugin({
      ...env
    }),
    new progressBarPlugin({
      clear: true
    }),
    new friendlyErrorsWebpackPlugin({
      clearConsole: true,
    }),
    new AutoDllPlugin({
      inject: true, // will inject the DLL bundles to index.html
      filename: '[name].[hash:8].js',
      path: `./assets/${pageName}/js/`,
      context: join(__dirname,'..','..'),
      plugins: [
        new UglifyJsPlugin({
          uglifyOptions: {
            compress: {
              warnings: false
            },
            mangle: {
              safari10: true
            }
          },
          parallel: true
        }),
      ],
      entry: {
        vendor: newdll
      }
    }),
  ])
}
